/* default box margin inside a Clamp. That's the main box */
.main-box { margin: 24px 12px; }

/* floating toolbar over map */
.map-toolbar { border-radius: 5px; margin: 0 16px 16px 0; }

.editbar {
	background-color: @accent_bg_color;
	color: @accent_fg_color;
}
